'use strict';

const axios = require('axios');
const urlJoin = require('url-join');
const semver = require('semver');

function getNpmInfo(npmName, registry) {
    if(!npmName) {
        return null
    }
    registry = registry || getDefaultRegistry();
    const npmInfoUrl = urlJoin(registry, npmName)
    return axios.get(npmInfoUrl)
        .then(res => {
            if(res.status === 200) {
                return res.data
            }else {
                return null
            }
        }).catch(() => {
            return Promise.reject(err)
        })
}

function getDefaultRegistry (isOriginal) {
    return isOriginal ? 'https://registry.npmjs.org': 'https://registry.npm.taobao.org'
}

async function getNpmVersions(npmName, registry) {
    const data = await getNpmInfo(npmName, registry)
    if(data) {
        return Object.keys(data.versions)
    } else {
        return []
    }
}

function getSemverVersion(baseVersion, versions) {
    return versions
        .filter(version => semver.satisfies(version, `^${baseVersion}`))
        .sort((a, b) => semver.gt(b, a))
}

async function getNpmSemverVersion(baseVersion, npmName, registry) {
    const versions = await getNpmVersions(npmName, registry)
    const newVersions = getSemverVersion(baseVersion, versions)
    if(newVersions?.length > 0) {
        return newVersions[0];
    }
    console.log("🚀 ~ file: index.js ~ line 47 ~ getNpmSemverVersion ~ newVersions", newVersions)
}

module.exports = {
    getNpmInfo,
    getNpmVersions,
    getNpmSemverVersion
};
